import { Component, Input, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { NbDialogService } from '@nebular/theme';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { User } from 'app/core/models/user';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { DialogConfirmationComponent } from 'app/shared/dialogs/dialog-confirmation/dialog-confirmation.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest } from 'rxjs';
import { takeUntil, flatMap, filter } from 'rxjs/operators';
import { Course } from '../../models/course';
import { CourseFile } from '../../models/course-file';
import { CourseFileService } from '../../services/course-file.service';
import { CourseService } from '../../services/course.service';

@Component({
  selector: 'ngx-course-file-list',
  templateUrl: './course-file-list.component.html',
  styleUrls: ['./course-file-list.component.scss']
})
export class CourseFileListComponent extends BaseComponent implements OnInit {
  @Input() course: Course;
  files: CourseFile[] = [];
  principal: SubatoPrincipal;
  deletingFile: CourseFile | null = null;

  loading = true;
  error: BackendError;

  constructor(
    private courseService: CourseService,
    private courseFileService: CourseFileService,
    private authService: AuthService,
    private dialogService: NbDialogService,
    private notificationService: NotificationService,
    private route: ActivatedRoute) {
    super();
  }

  ngOnInit(): void {
    this.refresh();
  }

  public refresh() {
    this.loading = true;
    combineLatest(this.authService.principal$, this.courseService.getFiles(this.course.id))
    .pipe(
      takeUntil(this.unsubscribe),
    )
    .subscribe(([principal, files]) => {
      this.principal = principal;
      this.files = files;

      this.loading = false;
    }, error => {
      if (BackendError.isBackendError(error)) {
        this.error = error;
        this.loading = false;
      } else {
        throw error;
      }
    });
  }

  downloadFile(file: CourseFile) {
    this.courseFileService.downloadFile(file)
    .subscribe(() => {}, error => {
      if (BackendError.isBackendError(error)) {
        this.notificationService.showError(
          error.format(
            "Fehler beim Herunterladen der Datei",
          )
        );
      } else {
        throw error;
      }
    })
  }

  deleteFile(file: CourseFile) {
    this.dialogService.open(DialogConfirmationComponent, {
      context: {
        title: `Löschung von Datei: ${file.name}`,
        text: `Bist du wirklich sicher, dass du die Datei '${file.name}' löschen willst?`,
        actionText: 'Löschen'
      }
    }).onClose.pipe(
      takeUntil(this.unsubscribe),
      filter(result => result && result.confirmed),
      flatMap(() => {
        this.deletingFile = file;
        var idx = this.files.indexOf(file);
        if(idx != -1) {
          this.files.splice(idx, 1);
        }
        
        return this.courseFileService.delete(file.id);
      })
    ).subscribe(() => {
      this.deletingFile = null;
      this.notificationService.showSuccess("Datei erfolgreich gelöscht");
    }, error => {
      this.deletingFile = null;

      if (BackendError.isBackendError(error)) {
        this.notificationService.showError(
          error.format(
            "Fehler beim Löschen der Datei",
          )
        );
      } else {
        throw error;
      }
    })
  }
}
