import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { ProgramModule } from 'app/features/modules/models/program-module';
import { ProgramModuleService } from 'app/features/modules/services/program-module.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { FormComponent } from 'app/shared/components/base/form.component';
import { NotificationService } from 'app/shared/services/notification.service';
import { Subject } from 'rxjs';
import { takeUntil, flatMap, debounceTime, filter } from 'rxjs/operators';
import { combineLatest, forkJoin, of } from 'rxjs';
import { AppStateService } from 'app/core/services/app.state.service';
import { Term } from 'app/features/modules/models/term';

@Component({
  selector: 'ngx-module-multiselect',
  templateUrl: './module-multiselect.component.html',
  styleUrls: ['./module-multiselect.component.scss']
})
export class ModuleMultiselectComponent extends BaseComponent implements OnInit {
  loading: boolean;
  term: Term;

  query: string;
  private queryChanged: Subject<string> = new Subject<string>();
  modules: ProgramModule[] = [];
  filteredModules: ProgramModule[] = [];
  private _selectedModules: ProgramModule[] = [];
  @Output() modulesChanged: EventEmitter<ProgramModule[]> = new EventEmitter<ProgramModule[]>();

  constructor(
    private programModuleService: ProgramModuleService,
    private notificationService: NotificationService,
    private appStateService: AppStateService
  ) { 
    super();
  }

  
  get selectedModules(): ProgramModule[] {
    return this._selectedModules;
  }

  @Input()
  set selectedModules(modules: ProgramModule[]) {
    this._selectedModules = [];
    modules.forEach(module => {
      if(this.modules.map(m => m.id).indexOf(module.id) != -1) {
        this.add(module, true); 
      }
    });
  }
  
  ngOnInit(): void {
    this.loading = true;

    combineLatest(this.appStateService.term$).pipe(
      takeUntil(this.unsubscribe),
      flatMap(([term]) => {
        this.term = term;

        return this.programModuleService.getAllForTerm(this.term?.id)
      }))
    .subscribe(modules => {
      this.modules = modules;
      this.filteredModules = modules;
      this.loading = false;
    });

    this.queryChanged
    .pipe(
      takeUntil(this.unsubscribe),
      debounceTime(500),
      flatMap(query => {
        this.loading = true;
        return this.programModuleService.getAllForTerm(this.term?.id, query)
      })
    )
    .subscribe(modules => {
      this.filteredModules = modules;
      this.loading = false;
    });
  }

  filterModules(query) {
    this.queryChanged.next(query);
  }

  get selectableModules() {
    return this.filteredModules.filter(m => !this.isModuleSelected(m))
  }

  isModuleSelected(module: ProgramModule) {
    return this.selectedModules.map(m => m.id).indexOf(module.id) != -1;
  }

  add(module: ProgramModule, skipEmit: boolean) {
    if (!this.isModuleSelected(module)) {
      this.selectedModules.push(module);

      if(!skipEmit) {
        this.modulesChanged.emit(this.selectedModules);
      }
    }
  }

  addAll() {
    this.filteredModules.forEach(m => this.add(m, true));
    this.modulesChanged.emit(this.selectedModules);
  }

  remove(module: ProgramModule) {
    this.selectedModules = this.selectedModules.filter(m => m != module);
    this.modulesChanged.emit(this.selectedModules);
  }
}
