import { HttpClient, HttpParams } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { BaseService } from "app/shared/services/base.service";
import { environment } from "environments/environment";
import { Observable } from "rxjs";
import { catchError, map } from "rxjs/operators";
import { Course } from "../models/course";
import { HttpCacheManager, withCache } from '@ngneat/cashew';
import { AppStateService } from "app/core/services/app.state.service";
import { UpdateCourseDto } from "../dto/update-course.dto";
import { BackendError } from "app/shared/models/backend-error";
import { CreateCourseDto } from "../dto/create-course.dto";
import { CourseStaffMembership } from "../models/course-staff-membership";
import { StaffGroup } from "../models/staff-group";
import { CourseUnit } from "../models/course-unit";
import { CourseFile } from "../models/course-file";

export interface CourseFilter {
    termId?: number;
    query?: string;
}

@Injectable({
    providedIn: 'root'
})
export class CourseService extends BaseService {
    constructor(private http: HttpClient, private appStateService: AppStateService, private cacheManager: HttpCacheManager) {
        super();
    }

    getById(id: number): Observable<Course> {
        return this.http.get<Course>(`${environment.backendUrl}/course/${id}`, {
            context: withCache() 
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(course => new Course(course))
        )
    }

    getAll(filter?: CourseFilter): Observable<Course[]> {
        let params = new HttpParams();
        if(filter?.termId != null) {
            params = params.set('termId', filter.termId);
        }

        if (filter?.query != null) {
            params = params.set("q", filter.query);
        }

        return this.http.get<Course[]>(`${environment.backendUrl}/course`, { 
            params: params,
            context: withCache({
                key: `courses-t${this.appStateService.getCurrentTerm()?.id ?? ''}-q${filter?.query ?? ''}`
            })
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(courses => courses.map(course => new Course(course)))
        )
    }
    
    create(dto: CreateCourseDto): Observable<Course> {
        return this.http.post<Course>(`${environment.backendUrl}/course`, dto).pipe(
            map(course => new Course(course))
        );
    }

    update(courseId: number, dto: UpdateCourseDto): Observable<Course> {
        return this.http.put<Course>(`${environment.backendUrl}/course/${courseId}`, dto).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(course => {
                this.cacheManager.clear();
                return new Course(course);
            })
        );
    }

    delete(id: number): Observable<void> {
        return this.http.delete<void>(`${environment.backendUrl}/course/${id}`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(() => this.cacheManager.clear())
        );
    }

    getUnits(id: number): Observable<CourseUnit[]> {
        return this.http.get<CourseUnit[]>(`${environment.backendUrl}/course/${id}/units`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(units => units.map(unit => new CourseUnit(unit)))
        )
    }

    getStaff(id: number): Observable<CourseStaffMembership[]> {
        return this.http.get<CourseStaffMembership[]>(`${environment.backendUrl}/course/${id}/staff`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(memberships => memberships.map(membership => new CourseStaffMembership(membership)))
        )
    }

    addStaff(id: number, userId: number, group: StaffGroup): Observable<CourseStaffMembership> {
        return this.http.put<CourseStaffMembership>(`${environment.backendUrl}/course/${id}/staff/${userId}/group/${group}`, {}).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(membership => {
                this.cacheManager.clear();
                return new CourseStaffMembership(membership);
            })
        );
    }

    removeStaff(id: number, userId: number, group: StaffGroup): Observable<void> {
        return this.http.delete<void>(`${environment.backendUrl}/course/${id}/staff/${userId}/group/${group}`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(() => this.cacheManager.clear())
        );
    }

    reimport(id: number): Observable<Course> {
        return this.http.post<Course>(`${environment.backendUrl}/course/${id}/reimport`, {}).pipe(
            map(course => new Course(course))
        );
    }

    getFiles(id: number): Observable<CourseFile[]> {
        return this.http.get<CourseFile[]>(`${environment.backendUrl}/course/${id}/files`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(files => files.map(file => new CourseFile(file)))
        )
    }
}
