import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { NotificationService } from 'app/shared/services/notification.service';

import { combineLatest, of } from 'rxjs';
import { takeUntil, flatMap } from 'rxjs/operators';
import { TaskInstance } from '../../../exercises/models/task-instance';
import { ExerciseService } from '../../../exercises/services/exercise.service';
import { BackendError } from 'app/shared/models/backend-error';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { AppStateService } from 'app/core/services/app.state.service';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { Solution } from '../../models/solution';
import { SolutionService } from '../../services/solution.service';
import { SolutionBreadcrumbBuilder } from './solution-breadcrumb-builder';

@Component({
  selector: 'ngx-solution',
  templateUrl: './solution.component.html',
  styleUrls: ['./solution.component.scss']
})
export class SolutionComponent extends BaseComponent implements OnInit {
  solution: Solution;
  taskInstances: TaskInstance[];
  principal: SubatoPrincipal;

  loading: boolean = true;
  justSubmitted = false;

  nextTaskInstance: TaskInstance | null = null;

  error: BackendError | null;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diese Lösung konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];

  constructor(
    private solutionService: SolutionService,
    private appStateService: AppStateService,
    private exerciseService: ExerciseService,
    private authService: AuthService,
    private notificationService: NotificationService,
    private route: ActivatedRoute,
    private router: Router) {
    super();
  }

  ngOnInit(): void {
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(([params, principal]) => {
          this.loading = true;
          this.principal = principal;
          this.justSubmitted = window.history.state && window.history.state.justSubmitted;

          let solutionId = Number(params.get("solution_id"));
          return this.solutionService.getById(solutionId);
        }),
        flatMap(solution => {
          this.solution = solution;
          this.breadcrumbs = SolutionBreadcrumbBuilder.getForSolution(solution);

          if(solution.exercise != null) {
            this.appStateService.setCourse(solution.exercise.course.id);
            return this.exerciseService.getTaskInstances(solution.exercise.id);
          } else {
            this.appStateService.setCourse(null);
            return of([]);
          }
        }))
      .subscribe((taskInstances) => {
        this.taskInstances = taskInstances;
        let taskInstance = taskInstances.find(ti => ti.task.id == this.solution.task.id);
        this.nextTaskInstance = taskInstance ? taskInstance.getNextTaskInstance(this.taskInstances) : null;
        this.loading = false;
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }

  get hasAccessToAcceptanceTests() {
    return this.solution.exercise ? this.principal.isLecturerOrAdmin(this.solution.exercise.course.id) : this.principal.hasStaffRole();
  }
}
