import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'environments/environment';
import { Observable, of } from 'rxjs';
import { catchError, map } from 'rxjs/operators';
import { BaseService } from 'app/shared/services/base.service';
import { HttpCacheManager, withCache } from '@ngneat/cashew';
import { BackendError } from 'app/shared/models/backend-error';
import { Language } from '../models/language';
import { CreateLanguageDto } from '../dto/create-language.dto';

@Injectable({
    providedIn: 'root'
})
export class LanguageService extends BaseService {
    constructor(private http: HttpClient, private cacheManager: HttpCacheManager) {
        super();
    }

    getById(id: string): Observable<Language> {
        return this.http.get<Language>(`${environment.backendUrl}/language/${id}`, {
            context: withCache()
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(lang => new Language(lang))
        )
    }

    getAll(): Observable<Language[]> {
        return this.http.get<Language[]>(`${environment.backendUrl}/language`, {
            context: withCache()
        })
        .pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(languages => languages.map(lang => new Language(lang)))
        );
    }

    create(dto: CreateLanguageDto): Observable<Language> {
        return this.http.post<Language>(`${environment.backendUrl}/language`, dto).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(lang => {
                this.cacheManager.clear();
                return new Language(lang);
            })
        );
    }

    update(langId: string, dto: CreateLanguageDto): Observable<Language> {
        return this.http.put<Language>(`${environment.backendUrl}/language/${langId}`, dto).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(lang => {
                this.cacheManager.clear();
                return new Language(lang);
            })
        );
    }

    delete(langId: string): Observable<void> {
        return this.http.delete<void>(`${environment.backendUrl}/language/${langId}`)
            .pipe(
                catchError(e => {
                    throw BackendError.parse(e);
                }),
            );
    }
}
