package de.hsrm.sls.subato.auth;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.modules.program.DegreeProgramRepository;
import de.hsrm.sls.subato.shared.helper.KeycloakRoleMapper;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.UserRepository;
import org.keycloak.KeycloakPrincipal;
import org.keycloak.adapters.springsecurity.token.KeycloakAuthenticationToken;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.cache.interceptor.KeyGenerator;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.stream.Collectors;

/**
 * Responsible for importing users authenticated by keycloak into the system, using the supplied token.
 *
 * Be careful: This implementation is not thread-safe, the method calling {@link #handleToken(KeycloakAuthenticationToken)}
 * should be declared as synchronized. Synchronization is not done in this class
 * because @Transactional does not work properly with synchronized
 */
@Service
@Transactional
public class KeycloakLoginHandler {
    @Autowired
    private PrincipalService principalService;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private DegreeProgramRepository degreeProgramRepository;

    private static Logger logger = LoggerFactory.getLogger(KeycloakLoginHandler.class);

    @Bean
    public KeyGenerator keycloakAuthKeyGenerator() {
        return (target, method, params) -> {
            var auth = (KeycloakAuthenticationToken) params[0];
            var alias = getAlias(auth);
            if(alias != null) {
                return alias;
            }

            return getUsername(auth);
        };
    }

    /**
     * if not cached, the user should be imported or updated (if exists) from the token
     * otherwise the cached principal for that user is returned
     *
     * Caching may be seen as redundant with the one in {@link PrincipalService#loadUserByUsername(String)}.
     * However, user details (the principal) can be loaded without a token, so it is necessary to do that.
     * In integration tests (for example) the user is loaded directly (without a keycloak token).
     *
     * caching here will prevent the user from getting updated from the token on every request
     */
    @Cacheable(value = "currentUser", keyGenerator = "keycloakAuthKeyGenerator")
    public SubatoPrincipal handleToken(KeycloakAuthenticationToken auth) {
        String alias = getAlias(auth);
        String username = alias;

        if(alias != null) {
            logger.info(String.format("Logged in with alias user, skipping import/update..."));
        } else {
            username = getUsername(auth);
            var mail = getMail(auth);

            var user = userRepository.findByLoginNameIgnoreCase(username).orElse(null);
            if(user == null && mail != null) {
                logger.info(String.format("User with loginName %s not found, trying email %s", username, mail));
                user = userRepository.findByMailIgnoreCase(getMail(auth)).orElse(null);
            }

            if (user == null) {
                logger.info("User not found, importing now...");
                user = new User();
            }

            updateUserFromToken(user, auth);
        }

        return (SubatoPrincipal) principalService.loadUserByUsername(username);
    }

    private User updateUserFromToken(User user, KeycloakAuthenticationToken auth) {
        var principal = (KeycloakPrincipal) auth.getPrincipal();
        var token = principal.getKeycloakSecurityContext().getToken();

        user.setLoginName(token.getPreferredUsername());
        user.setFirstName(token.getGivenName());
        user.setLastName(token.getFamilyName());
        user.setMail(token.getEmail());
        user.setActive(true);
        user.setKeycloakId(principal.getName());
        user.setProgram(null);

        if(token.getOtherClaims() != null && token.getOtherClaims().containsKey("stg")) {
            var posCode = (String) token.getOtherClaims().get("stg");
            var program = degreeProgramRepository.findByPosCode(posCode);
            user.setProgram(program);
        }

        user.getRoles().clear();
        user = userRepository.save(user);

        var roles = auth.getAccount().getRoles().stream().map(r -> KeycloakRoleMapper.getRole(r))
                .filter(r -> r != null)
                .collect(Collectors.toList());
        user.getRoles().addAll(roles);
        user = userRepository.save(user);
        logger.info("Imported/updated user %s".formatted(user.getLoginName()));

        return user;
    }

    private String getUsername(KeycloakAuthenticationToken auth) {
                var principal = (KeycloakPrincipal) auth.getPrincipal();
        var token = principal.getKeycloakSecurityContext().getToken();
        return token.getPreferredUsername();
    }

    private String getMail(KeycloakAuthenticationToken auth) {
        var principal = (KeycloakPrincipal) auth.getPrincipal();
        var token = principal.getKeycloakSecurityContext().getToken();
        return token.getEmail();
    }

    private String getAlias(KeycloakAuthenticationToken auth) {
        var principal = (KeycloakPrincipal) auth.getPrincipal();
        var token = principal.getKeycloakSecurityContext().getToken();
        if(token.getOtherClaims() != null && token.getOtherClaims().containsKey("alias")) {
            return (String) token.getOtherClaims().get("alias");
        }

        return null;
    }
}
