package de.hsrm.sls.subato.auth.policy.exercises;

import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.auth.principal.CourseStaffMembership;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.auth.policy.courses.CourseViewPolicy;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.courses.course.CourseVisibility;
import de.hsrm.sls.subato.courses.course.Course_;
import de.hsrm.sls.subato.courses.membership.CourseStaffMembership_;
import de.hsrm.sls.subato.courses.membership.StaffGroup;
import de.hsrm.sls.subato.courses.unit.CourseUnit_;
import de.hsrm.sls.subato.exercises.exercise.Exercise;
import de.hsrm.sls.subato.exercises.exercise.Exercise_;
import de.hsrm.sls.subato.user.User_;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;

import javax.persistence.criteria.JoinType;
import java.time.LocalDateTime;

/**
 * The current user is allowed to view an exercise within a certain course
 */
@Component
public class ExerciseViewPolicy implements PolicyRule {
    @Autowired
    @Lazy
    private PolicyEvaluator policy;

    @Override
    public boolean check(CheckPolicyRequest request) {
        if(!request.matches(ExercisesAction.EXERCISE_VIEW)) {
            return false;
        }

        var exercise = (Exercise) request.getObj();
        var canViewCourse = policy.check(CheckPolicyRequest.fromPrincipal(exercise.getCourse(), CoursesAction.COURSE_VIEW));
        return canViewCourse && (exercise.isVisible() || request.getSubject() != null && request.getSubject().hasGroup(exercise.getCourse().getId(), StaffGroup.LECTURER));
    }

    @Override
    public Specification checkAsSpec(CheckPolicyRequest request) {
        if(!request.matches(ExercisesAction.EXERCISE_VIEW)) {
            return Specification.where(null);
        }
        return isCourseVisible(request.getSubject()).and(isVisible().or(hasLecturerGroup(request.getSubject())));
    }

    /**
     * copied from {@link CourseViewPolicy#isVisible(SubatoPrincipal)} and adapted to Exercise entity
     * sorry, I don't know how to reuse it yet.
     * todo
     */
    public static Specification<Exercise> isCourseVisible(SubatoPrincipal principal) {
        return (ex, cq, cb) -> {
            var participants = ex.join(Exercise_.course)
                    .join(Course_.units, JoinType.LEFT)
                    .join(CourseUnit_.participants, JoinType.LEFT);

            cq.distinct(true);

            return cb.or(
                    cb.equal(ex.get(Exercise_.course).get(Course_.visibility), CourseVisibility.PUBLIC),
                    cb.and(
                            cb.equal(ex.get(Exercise_.course).get(Course_.visibility), CourseVisibility.INTERNAL),
                            cb.equal(cb.literal(true), principal != null)
                    ),
                    cb.and(
                            cb.equal(ex.get(Exercise_.course).get(Course_.visibility), CourseVisibility.PARTICIPANTS),
                            cb.equal(cb.literal(true), principal != null),
                            cb.equal(participants.get(User_.id), principal != null ? principal.getId() : -1)
                    ),
                    cb.and(
                            cb.equal(ex.get(Exercise_.course).get(Course_.visibility), CourseVisibility.PRIVATE),
                            cb.equal(cb.literal(true), principal != null && principal.hasRole(Role.Staff))
                    )
            );
        };
    }

    public static Specification<Exercise> isVisible() {
        var now = LocalDateTime.now();

        return (ex, cq, cb) -> cb.or(
                cb.isNull(ex.get(Exercise_.visibleFrom)),
                cb.lessThan(ex.get(Exercise_.visibleFrom), now)
        );
    }

    public static Specification<Exercise> hasLecturerGroup(SubatoPrincipal principal) {
        return (ex, cq, cb) -> {
            var memberships = ex.join(Exercise_.course).join(Course_.memberships, JoinType.LEFT);
            cq.distinct(true);
            return cb.and(
                    cb.equal(memberships.get(CourseStaffMembership_.user).get(User_.id), principal != null ? principal.getId() : -1),
                    cb.equal(memberships.get(CourseStaffMembership_.group), StaffGroup.LECTURER)
            );
        };
    }
}
