package de.hsrm.sls.subato.shared.error;

import com.fasterxml.jackson.annotation.JsonAutoDetect;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.ANY;

/**
 * An immutable data structure representing HTTP error response bodies. JSON
 * representation of this class would be something like the following:
 * <pre>
 *     {
 *         "status_code": 404,
 *         "reason_phrase": "Not Found",
 *         "error_code": 2
 *     }
 * </pre>
 *
 * @author Ali Dehghani
 */
@JsonAutoDetect(fieldVisibility = ANY)
public class ErrorResponse {
    /**
     * The 4xx or 5xx status code for error cases, e.g. 404
     */
    private final int statusCode;

    /**
     * The HTTP reason phrase corresponding the {@linkplain #statusCode},
     * e.g. Not Found
     *
     * @see <a href="https://www.w3.org/Protocols/rfc2616/rfc2616-sec6.html">
     * Status Code and Reason Phrase</a>
     */
    private final String reasonPhrase;

    /**
     * application-level error code.
     *
     * Using these errors we provide more information about the
     * actual error
     */
    private final ErrorCode errorCode;

    private final Object data;

    public ErrorResponse(int statusCode, String reason, ErrorCode errorCode, Object data) {
        this.statusCode = statusCode;
        this.reasonPhrase = reason;
        this.errorCode = errorCode;
        this.data = data;
    }

    public ErrorResponse(int statusCode, String reasonPhrase, ErrorCode errorCode) {
        this(statusCode, reasonPhrase, errorCode, null);
    }

    @Override
    public String toString() {
        return "ErrorResponse{" +
                "statusCode=" + statusCode +
                ", reasonPhrase='" + reasonPhrase + '\'' +
                ", errorCode=" + errorCode +
                ", data=" + data +
                '}';
    }
}