package de.hsrm.sls.subato.solutions.evaluation;

import de.hsrm.sls.subato.shared.helper.FileHelper;
import de.hsrm.sls.subato.solutions.evaluation.evaluator.Evaluator;
import de.hsrm.sls.subato.solutions.evaluation.evaluator.EvaluatorService;
import de.hsrm.sls.subato.solutions.evaluation.request.EvalRequestGenerator;
import de.hsrm.sls.subato.solutions.evaluation.result.EvalResult;
import de.hsrm.sls.subato.solutions.evaluation.result.StyleCheckResult;
import de.hsrm.sls.subato.solutions.evaluation.result.TestCaseResult;
import de.hsrm.sls.subato.solutions.solution.Solution;
import de.hsrm.sls.subato.solutions.solution.SolutionContentRepository;
import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.tasks.task.TaskContentRepository;
import de.hsrm.sls.subato.tasks.task.TaskRepository;
import de.hsrm.sls.subato.testexec.EvaluatorService.EvaluatorServiceGrpc;
import de.hsrm.sls.subato.testexec.EvaluatorService.TestResourceFile;
import de.hsrm.sls.subato.testexec.EvaluatorService.TestResult;
import de.hsrm.sls.subato.testexec.EvaluatorService.TestSet;
import io.grpc.Status;
import io.grpc.netty.NegotiationType;
import io.grpc.netty.NettyChannelBuilder;
import io.micrometer.core.instrument.Counter;
import io.micrometer.core.instrument.MeterRegistry;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

@Service
public class SolutionEvaluationService {
    @Autowired
    private EvaluatorConfig evaluatorConfig;

    @Autowired
    private EvalRequestGenerator requestGenerator;

    @Autowired
    private SolutionContentRepository solutionContentRepository;

    @Autowired
    private TaskContentRepository taskContentRepository;

    @Autowired
    private EvaluatorService evaluatorService;

    @Autowired
    private TaskRepository taskRepository;

    private final Counter evaluations;
    private final Counter failedEvaluations;
    private final Counter timeoutEvaluations;
    private final AtomicInteger pendingEvaluations;

    public SolutionEvaluationService(MeterRegistry meterRegistry) {
        timeoutEvaluations = meterRegistry.counter("subato.timeout_evaluations");
        evaluations = meterRegistry.counter("subato.evaluations");
        failedEvaluations = meterRegistry.counter("subato.failed_evaluations");
        pendingEvaluations = meterRegistry.gauge("subato.pending_evaluations", new AtomicInteger(0));
    }

    /**
     * Evaluates the solution with regular tests
     *
     * @see #test(Solution, boolean)
     */
    public EvalResult test(Solution solution) {
        return test(solution, false);
    }

    /**
     * @param solution solution to evaluate
     * @param acceptanceCheck if the special acceptance tests (X)OR if the regular tests should be used for evaluation.
     * @return result of the evaluation, returned by the evalautor
     * @throws EvaluationFailedException if the evaluation failed
     * @throws UnsupportedOperationException  if the task does not support running tests
     */
    public EvalResult test(Solution solution, boolean acceptanceCheck) {
        Task task = solution.getTask();
        var canEvaluate = !acceptanceCheck && task.evaluationPossible() || acceptanceCheck && task.isAcceptanceTestAvailable();
        if(!canEvaluate) {
            throw new UnsupportedOperationException("Cannot evaluate solution %s because evaluator or test files are missing".formatted(solution.getId()));
        }

        var evaluator = refreshEvaluator(task);
        var evalRequest = requestGenerator.generateRequest(solution, acceptanceCheck);
        var resourceFiles = evalRequest.getFilesToTransmit().stream()
                .map(f -> {
                    var file = taskContentRepository.getFileByRef(task, f);
                    return TestResourceFile.newBuilder().setFileName(f.getPath()).setContent(FileHelper.readString(file)).build();
                }).collect(Collectors.toList());


        var codeFile = solutionContentRepository.getFile(solution, evalRequest.getCodeFile().getUuid());
        var testSet = TestSet.newBuilder()
                .addAllTestFiles(resourceFiles)
                .setCode(FileHelper.readString(codeFile))
                .setCodeFqdn(evalRequest.getTemplateFile().getPath())
                .setTestImage(evaluator.getReference())
                .setCheckStyle(evaluator.isStyle())
                .setIsApprovalCheck(acceptanceCheck)
                .build();

        try {
            evaluations.increment();
            pendingEvaluations.incrementAndGet();

            var channel = NettyChannelBuilder.forAddress(evaluatorConfig.getUrl(), evaluatorConfig.getPort()).negotiationType(NegotiationType.PLAINTEXT).build();
            var stub = EvaluatorServiceGrpc.newBlockingStub(channel).withDeadlineAfter(evaluatorConfig.getTimeout(), TimeUnit.MILLISECONDS);

            var result = stub.runTest(testSet);

            channel.shutdownNow();
            channel.awaitTermination(40, TimeUnit.SECONDS);

            return buildResult(result, evaluator);
        } catch(io.grpc.StatusRuntimeException e) {
            failedEvaluations.increment();

            var isTimeout = e.getStatus().getCode() == Status.Code.DEADLINE_EXCEEDED;
            if(isTimeout) {
                timeoutEvaluations.increment();
            }

            throw new EvaluationFailedException(e);
        } catch(Exception e) {
            failedEvaluations.increment();
            throw new EvaluationFailedException(e);
        } finally {
            pendingEvaluations.decrementAndGet();
        }
    }

    private Evaluator refreshEvaluator(Task task) {
        var evaluator = evaluatorService.refresh(task.getEvaluator());
        task.setEvaluator(evaluator);
        taskRepository.save(task);
        return evaluator;
    }

    private EvalResult buildResult(TestResult result, Evaluator evaluator) {
        var evalResult = new EvalResult();
        evalResult.setEvaluator(evaluator);
        evalResult.setTestsRun(result.getTests());
        evalResult.setTestFailures(result.getFailures());

        evalResult.setTestErrors(result.getErrors());
        evalResult.setCompiling(!result.getCompilationFailed());
        evalResult.setCompilationOutput(fixNullBytes(result.getCompilationOutput()));

        evalResult.setFrees(result.getFrees());
        evalResult.setAllocations(result.getAllocations());

        evalResult.setRuntimeOutput(fixNullBytes(result.getRuntimeOutput()));

        var testCaseResults = result.getTestCasesList().stream().map(testCase -> {
            var testCaseResult = new TestCaseResult();
            testCaseResult.setName(testCase.getName());
            testCaseResult.setClassName(testCase.getClassName());
            testCaseResult.setText(testCase.getFailure().getText());
            testCaseResult.setType(testCase.getFailure().getType());
            testCaseResult.setMessage(testCase.getFailure().getMessage());
            testCaseResult.setEvalResult(evalResult);

            return testCaseResult;
        }).collect(Collectors.toList());
        evalResult.getTestCaseResults().addAll(testCaseResults);

        var styleCheckResults = result.getStyleCheckResultsList().stream().map(styleCheck -> {
            var styleCheckResult = new StyleCheckResult();
            styleCheckResult.setFile(styleCheck.getFile());
            styleCheckResult.setMessage(styleCheck.getMessage());
            styleCheckResult.setLine(styleCheck.getLine());
            styleCheckResult.setCol(styleCheck.getColumn());
            styleCheckResult.setSeverity(styleCheck.getSeverity());
            styleCheckResult.setEvalResult(evalResult);

            return styleCheckResult;
        }).collect(Collectors.toList());
        evalResult.getStyleCheckResults().addAll(styleCheckResults);

        return evalResult;
    }

    /**
     * Evaluator gibt in Kompilierungsausgaben und anderen Ausgaben ggf. Null-bytes zurück, die
     * Postgres nicht in Textfeldern unterstützt, deshalb ersetzen
     * @param s
     * @return
     */
    private String fixNullBytes(String s) {
        return s.replaceAll("\u0000", "");
    }
}
