package de.hsrm.sls.subato.tasks.feedback;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.tasks.task.TaskNotFoundException;
import de.hsrm.sls.subato.tasks.task.TaskService;
import de.hsrm.sls.subato.user.UserNotFoundException;
import de.hsrm.sls.subato.user.UserService;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Responsible for CRUD operations regarding {@link TaskFeedback}
 */
@Service
public class TaskFeedbackService {
    @Autowired
    private TaskFeedbackRepository feedbackRepository;

    @Autowired
    private PrincipalService principalService;

    @Autowired
    private UserService userService;

    @Autowired
    private TaskService taskService;

    /**
     * @throws TaskFeedbackNotFoundException
     */
    public TaskFeedback markAsRead(long taskId, long userId) {
        var feedback = getFeedbackForTaskByUser(taskId, userId);
        feedback.setRead(true);
        return feedbackRepository.save(feedback);
    }

    /**
     * @throws TaskFeedbackNotFoundException
     */
    public TaskFeedback getFeedbackForTaskByUser(long taskId, long userId) {
        return feedbackRepository.findByTaskIdAndAndFromId(taskId, userId).orElseThrow(() -> new TaskFeedbackNotFoundException());
    }

    /**
     * gets all unchecked feedback entries where a comment is present for a given task
     * @throws TaskNotFoundException
     */
    public List<TaskFeedback> getPendingFeedbackForTask(long taskId) {
        var task = taskService.findById(taskId);
        return task.getFeedback().stream()
                .filter(f -> !f.isRead())
                .filter(f -> !Strings.isEmpty(f.getComment()))
                .collect(Collectors.toList());
    }

    /**
     * idempotent create/update operation for feedback entries
     * @throws TaskNotFoundException
     * @throws UserNotFoundException
     */
    public TaskFeedback createOrUpdate(CreateTaskFeedbackDto dto, long taskId) {
        var currentUser = principalService.getPrincipal();
        var user = userService.findById(currentUser.getId());
        var task = taskService.findById(taskId);

        TaskFeedback feedback;

        try {
            feedback = getFeedbackForTaskByUser(taskId, currentUser.getId());
        } catch(TaskFeedbackNotFoundException ex) {
            feedback = new TaskFeedback();
            feedback.setTask(task);
        }

        feedback.setComment(dto.getComment());
        feedback.setComplexity(dto.getComplexity());
        feedback.setComprehensibility(dto.getComprehensibility());
        feedback.setFrom(user);
        feedback.setSubmittedAt(LocalDateTime.now());
        feedback.setRead(Strings.isEmpty(dto.getComment()));

        feedback = feedbackRepository.save(feedback);
        return feedback;
    }
}
