package de.hsrm.sls.subato.tasks.pool;

import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.tasks.pool.sync.repository.TaskPoolRepositoryManager;
import de.hsrm.sls.subato.tasks.imp.log.TaskImportLog;
import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.tasks.task.TaskFilter;
import de.hsrm.sls.subato.tasks.task.TaskService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * responsible for CRUD operations regarding task pools
 */
@Service
public class TaskPoolService {
    @Autowired
    private TaskPoolRepository taskPoolRepository;

    @Autowired
    private TaskService taskService;

    @Autowired
    private TaskPoolRepositoryManager taskPoolRepositoryManager;

    /**
     * @throws TaskPoolNotFoundException
     */
    public TaskPool findById(long id) {
        return taskPoolRepository.findById(id).orElseThrow(() -> new TaskPoolNotFoundException(id));
    }

    public List<TaskPool> getAll() {
        return taskPoolRepository.findAll();
    }

    public List<TaskPool> getAllWithRepository() {
        return taskPoolRepository
                .findAll()
                .stream()
                .filter(pool -> pool.isLinkedToRepository())
                .collect(Collectors.toList());
    }

    public Page<Task> getTasks(long id, Pageable page, Optional<String> query) {
        var pool = findById(id);
        var filter = new TaskFilter().poolId(id).query(query.orElse(null));
        return taskService.getAll(page, filter);
    }

    /**
     *
     * @throws TaskPoolNotFoundException
     */
    public List<TaskImportLog> getLogs(long id) {
        var pool = findById(id);
        var logs = pool.getLog();
        logs.sort(Comparator.comparing(TaskImportLog::getAt).reversed());
        return logs;
    }

    private void updateTaskPool(TaskPool pool, CreateTaskPoolDto dto) {
        pool.setDescription(dto.getDescription());
        pool.setName(dto.getName());
        pool.setRepositoryUrl(dto.getRepositoryUrl());
    }

    /**
     * @throws ConstraintViolationException if invalid
     */
    private void validateTaskPool(TaskPool pool) {
        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(pool);
        if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
        }
    }

    /**
     * @param dto attributes for the new pool
     * @return created pool
     * @throws ConstraintViolationException
     */
    @Transactional
    public TaskPool create(CreateTaskPoolDto dto) {
        var taskPool = new TaskPool();
        updateTaskPool(taskPool, dto);
        validateTaskPool(taskPool);

        var pool = taskPoolRepository.save(taskPool);

        if(taskPool.isLinkedToRepository()) {
            taskPoolRepositoryManager.testRepository(pool);
        }

        return pool;
    }

    /**
     * @param id pool to be changed
     * @param dto changes
     * @return changed pool
     * @throws ConstraintViolationException
     * @throws TaskPoolNotFoundException
     */
    public TaskPool update(long id, CreateTaskPoolDto dto) {
        var taskPool = findById(id);
        var wasLinkedToRepository = taskPool.isLinkedToRepository();
        var oldRepositoryUrl = taskPool.getRepositoryUrl();

        updateTaskPool(taskPool, dto);
        validateTaskPool(taskPool);

        if(taskPool.isLinkedToRepository()) {
            taskPoolRepositoryManager.testRepository(taskPool);
        }

        if(wasLinkedToRepository && !oldRepositoryUrl.equals(taskPool.getRepositoryUrl())) {
            taskPoolRepositoryManager.removeRepository(taskPool);
        }

        return taskPoolRepository.save(taskPool);
    }

    /**
     * gets all courses having at least one task from the given task pool
     * @param id of the pool
     * @return courses
     */
    public List<Course> getCourses(long id) {
        var pool = findById(id);
        // todo: improve performance
        var courses = pool.getTasks().stream()
                .map(t -> t.getInstances()).flatMap(Collection::stream)
                .map(ti -> ti.getExercise().getCourse())
                .distinct()
                .collect(Collectors.toList());

        courses.sort((a,b) -> a.getTerm() == null ? -1 : b.getTerm() == null ? 1 : (int) (b.getTerm().getId() - a.getTerm().getId()));
        return courses;
    }

    /**
     * deletes a task pool, including all tasks and linked task instances, solutions etc. (DANGEROUS, BE CAREFUL!)
     */
    public void delete(long id) {
        var pool = findById(id);
        this.taskPoolRepository.delete(pool);

        if(pool.isLinkedToRepository()) {
            taskPoolRepositoryManager.removeRepository(pool);
        }
    }
}
