package de.hsrm.sls.subato.tasks.stef.parser.meta;

import de.hsrm.sls.subato.shared.fs.FileTree;
import de.hsrm.sls.subato.tasks.stef.Meta;
import de.hsrm.sls.subato.tasks.stef.file.FileRef;
import de.hsrm.sls.subato.tasks.stef.file.FileType;
import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.TaskXml;
import org.springframework.stereotype.Service;

import java.nio.file.Path;
import java.util.ArrayList;

@Service
public class FileRefsExtractor {
    private static final String FILES_DIR = "files";

    /**
     * resolves and validates file references we are interested in
     * @param doc XML document representing the meta.xml
     * @param meta
     * @param fileRoot points to the spec in the file system
     * @throws InvalidSpecException if file reference is invalid, i.e. if the file could not be resolved or the decl is invalid
     */
    public void extract(TaskXml doc, Meta meta, String fileRoot) {
        var fileRefs = new ArrayList<FileRef>();
        if(doc.getFiles() != null && doc.getFiles().getFiles() != null) {
            for (var fNode : doc.getFiles().getFiles()) {
                var typeAttr = fNode.getType() != null ? fNode.getType() : "";
                FileType type = FileType.fromIdentifier(typeAttr);

                if (type == null) {
                    type = FileType.MISC;
                }

                var isPublic = fNode.isPublic() != null ? fNode.isPublic().booleanValue() : false;
                var desc = fNode.getDescription();

                var filePath = fNode.getPath();
                if (filePath == null) {
                    throw new InvalidSpecException("Nicht alle in meta.xml deklarierten Dateien haben einen Namen/Pfad");
                }

                filePath = filePath.startsWith("/") ? filePath.substring(1) : filePath;
                filePath = filePath.startsWith("./") ? filePath.substring(2) : filePath;

                var targetFolder = Path.of(FILES_DIR).resolve(type.getIdentifier());
                var actualFilePath = Path.of(fileRoot).resolve(targetFolder).resolve(filePath);
                var basePath = targetFolder.resolve(filePath).getParent();

                var f = FileTree.from(actualFilePath.toFile(), basePath != null ? basePath.toString() : "", false, true);

                if (f == null) {
                    throw new InvalidSpecException("Datei %s in meta.xml deklariert aber nicht im Ordner %s gefunden".formatted(filePath, targetFolder));
                }

                var ref = new FileRef();
                ref.setPath(filePath);
                ref.setFile(f);
                ref.setPublic(isPublic);
                ref.setDescription(desc);
                ref.setType(type);

                fileRefs.add(ref);
            }
        }

        meta.setFiles(fileRefs);
    }
}
