package de.hsrm.sls.subato.tasks.task;

import de.hsrm.sls.subato.tasks.stef.file.FileRef;
import de.hsrm.sls.subato.tasks.pool.TaskPoolReadyOnlyException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.util.Comparator;
import java.util.List;
import java.util.UUID;

import static de.hsrm.sls.subato.tasks.task.TaskSpecifications.archived;

/**
 * responsible for read operations regarding tasks, such as getting tasks, their files, file content etc.
 */
@Service
public class TaskService {
    @Autowired
    private TaskRepository taskRepository;

    @Autowired
    private TaskContentRepository taskContentRepository;


    /**
     * @throws TaskNotFoundException
     */
    public Task findById(long id) {
        var task = taskRepository.findById(id).orElse(null);
        if(task == null || task.isArchived()) {
            throw new TaskNotFoundException(id);
        }

        return task;
    }

    /**
     * gets all tasks
     * @return
     */
    public Page<Task> getAll(Pageable page, TaskFilter filter) {
        Specification<Task> spec = filter.toSpecification();
        spec = spec.and(archived(false));

        return taskRepository.findAll(spec, page);
    }

    /**
     * returns the STEF specification of the task as zip file
     * @param task
     * @return
     */
    File getSpecAsZip(Task task) {
        return taskContentRepository.getSpecAsZip(task);
    }

    /**
     * gets a file within the text-folder in the STEF spec.
     *
     * can be used for resolving images referenced in the task content.
     *
     * @param task task from where to get the file
     * @param path relative to the text folder in the spec. example: images/BTree.png refers to /15SomeTask/text/images/BTree.png
     * @return the file
     * @throws AssetNotFoundException if no file exists in the spec with the given relative path
     */
    public File getTextAsset(Task task, String path) {
        var file = taskContentRepository.getFile(task, Path.of("text").resolve(path));
        if(!file.exists()) {
            throw new AssetNotFoundException("asset %s not found for task %s".formatted(path, task.getId()));
        }

        try {
            // check for malicious attacks, e.g. path = '../files/solution/First.java'
            var textFolder = taskContentRepository.getTaskPath(task).resolve("text");
            if(!file.getCanonicalPath().startsWith(textFolder.toFile().getCanonicalPath() + File.separator)) {
                throw new AssetNotFoundException("asset %s not found for task %s".formatted(path, task.getId()));
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        return file;
    }

    /**
     * gets all files associated with the task, independent of whether they are visible and sorted by their type
     */
    public List<FileRef> getFiles(Task task) {
        var files = taskContentRepository.getFileRefs(task);

        files.sort(Comparator.comparingInt(a -> a.getType().getOrder()));
        return files;
    }

    public FileRef getFileRefById(Task task, String uuid) {
        return taskContentRepository.getFileRefById(task, UUID.fromString(uuid)).orElseThrow(() -> new FileNotFoundException(uuid));
    }

    public File getFile(Task task, String uuid) {
        var ref = getFileRefById(task, uuid);
        return taskContentRepository.getFileByRef(task, ref);
    }

    /**
     * archive (soft-delete) the task, so that links to solutions, task instances etc. will be preserved
     * @param task to be archived
     * @return the archived task
     * @throws TaskPoolReadyOnlyException if the pool cannot be manipulated directly
     * @throws TaskNotFoundException
     */
    public Task archive(Task task) {
        if(task.getPool().isLinkedToRepository()) {
            throw new TaskPoolReadyOnlyException();
        }

        task.setArchived(true);
        return taskRepository.save(task);
    }

    /**
     * @see #archive(Task)
     */
    public Task archive(long id) {
        var task = findById(id);
        return archive(task);
    }
}
