package de.hsrm.sls.subato.solutions.evaluation;

import de.hsrm.sls.subato.utils.SubatoDataDirCleanup;
import de.hsrm.sls.subato.utils.SubatoTmpDirCleanup;
import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.shared.fs.FileContent;
import de.hsrm.sls.subato.solutions.solution.Solution;
import de.hsrm.sls.subato.solutions.solution.SolutionContentRepository;
import de.hsrm.sls.subato.solutions.solution.SolutionRepository;
import de.hsrm.sls.subato.tasks.pool.TaskPoolRepository;
import de.hsrm.sls.subato.tasks.pool.sync.TaskPoolSyncService;
import de.hsrm.sls.subato.tasks.stef.file.FileType;
import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.tasks.task.TaskService;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * Integration test for evaluating generated solutions via Eva.
 * For every task in the STEF examples pool a solution is generated and evaluated
 *
 * Remote dependencies:
 * - GitLab (access to repository of the task pool)
 * - Eva (resolving evaluators of tasks)
 */
@SpringBootTest
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
@AutoConfigureTestDatabase(replace = AutoConfigureTestDatabase.Replace.ANY)
@Tag(Tags.INTEGRATION)
@Tag(Tags.EXTERNAL_DEPENDENCY)
@Transactional
public class SolutionEvaluationTests {
    private static Logger logger = LoggerFactory.getLogger(SolutionEvaluationTests.class);

    @Autowired
    TaskPoolSyncService syncService;

    @Autowired
    TaskPoolRepository poolRepository;

    @Autowired
    TaskService taskService;

    @Autowired
    SolutionRepository solutionRepository;

    @Autowired
    SolutionContentRepository solutionContentRepository;

    @Autowired
    SolutionEvaluationService evaluationService;

    @BeforeAll
    public static void beforeAll(@Autowired SubatoDataDirCleanup dataDirCleanup, @Autowired SubatoTmpDirCleanup tmpDirCleanup) {
        dataDirCleanup.exec();
        tmpDirCleanup.exec();
    }

    @Test
    public void shouldEvaluateSolutions() {
        syncService.syncPool(poolRepository.findById(1L).get(), true);
        var pool = poolRepository.findById(1L).get();

        for(var task : pool.getTasks()) {
            Solution finalSolution = createDefaultSolution(task);
            if(finalSolution == null) {
                continue;
            }

            try {
                var result = evaluationService.test(finalSolution);
                logger.info("Evaluating solution for %s: ok".formatted(task.getIdName()));
            } catch(EvaluationFailedException ex) {
                logger.error("Evaluating solution for %s: failed".formatted(task.getIdName()), ex);
            }
        }
    }

    private Solution createDefaultSolution(Task task) {
        var taskFiles = taskService.getFiles(task);
        var templateFileRef = taskFiles.stream().filter(f -> f.getType() == FileType.TEMPLATE).findFirst().orElse(null);
        if(templateFileRef != null) {
            var templateFile = taskService.getFile(task, templateFileRef.getFile().getUuid().toString());

            var solution = new Solution();
            solution.setTask(task);
            solution.setSubmittedAt(LocalDateTime.now());
            solution = solutionRepository.save(solution);
            solutionContentRepository.store(FileContent.fromFile(templateFile), solution);
            return solution;
        } else {
            return null;
        }
    }
}
